==> migrate_wizard-1.0.16-alpha1.tar.gz: migrate_wizard/src/MWManageDataService.php <==
<?php

namespace Drupal\migrate_wizard;

use Drupal\Core\Language\LanguageInterface;
use Drupal\migrate\Plugin\MigrationDeriverTrait;
use Drupal\migrate_drupal\Plugin\migrate\source\d7\FieldableEntity;
use Drupal\user\Entity\Role;
use Drupal\Component\Transliteration\PhpTransliteration;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ConfigManager;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ExtensionList;

/**
 * Defines a Service to update nodes.
 */
class MWManageDataService extends FieldableEntity{

  /**
   * {@inheritdoc}
   */
  protected $entityType;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected $migrationUrlFilesProd;

  /**
   * {@inheritdoc}
   */
  protected $extensionListModule;

  /**
   * {@inheritdoc}
   */
  protected $configManager;

  /**
   * {@inheritdoc}
   */
  protected $transliterator;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager,
                              EntityTypeManagerInterface $entity_type_manager,
                              ConfigFactoryInterface $configFactory,
                              ExtensionList $extension_list_module,
                              ConfigManager $config_manager,
                              PhpTransliteration $php_transliteration) {

    $this->config = $configFactory->getEditable('migrate_wizard.settings');
    if (is_array($this->config->get('migrate_wizard_settings'))) {
      $this->migrationUrlFilesProd = $this->config->get('migrate_wizard_settings')['migration_url_files_prod'];
    }
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->extensionListModule = $extension_list_module;
    $this->configManager = $config_manager;
    $this->transliterator = $php_transliteration;
  }

  use MigrationDeriverTrait;

  /**
   * {@inheritdoc}
   */
  protected function getNodeTypesQuery() {
    return static::getSourcePlugin('d7_node_type');
  }

  /**
   * {@inheritdoc}
   */
  protected function getVocabulariesQuery() {
    return static::getSourcePlugin('d7_taxonomy_vocabulary');
  }

  /**
   * {@inheritdoc}
   */
  protected function getFieldCollectionsQuery() {
    $query = $this->select('field_config', 'fc')
      ->fields('fc', ['field_name'])
      ->groupBy('fc.field_name')
      ->condition('fc.type', 'field_collection');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  protected function getParagraphsQuery() {
    return static::getSourcePlugin('d7_paragraphs_item');
  }
  /**
   * {@inheritdoc}
   */
  public function readOriginNodes(): void
  {
    $results = $this->getNodeTypesQuery();
    foreach ($results as $result) {
      // Check if entity exists.
      $entity = $this->entityTypeManager->getStorage('origin_node')->load($result->getSourceProperty('type'));
      if (!$entity) {
        $entity = $this->entityTypeManager->getStorage('origin_node')->create([
          'id' => $result->getSourceProperty('type'),
          'label' => $result->getSourceProperty('name'),
          'description' => $result->getSourceProperty('description'),
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginNodes(): void
  {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_node')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_node')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginVocabularies(): void
  {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_vocabulary')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_vocabulary')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function readOriginVocabularies(): void
  {
    $results = $this->getVocabulariesQuery();

    foreach($results as $result){
     $entity = $this->entityTypeManager->getStorage('origin_vocabulary')->load($result->getSourceProperty('machine_name'));
      if ($entity == NULL) {
        $entity = $this->entityTypeManager->getStorage('origin_vocabulary')->create([
          'id' => $result->getSourceProperty('machine_name'),
          'label' => $result->getSourceProperty('name'),
          'description' => $result->getSourceProperty('description'),
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginFieldCollections(): void
  {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_field_collection')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_field_collection')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function readOriginFieldCollections(): void
  {
    $query = $this->getFieldCollectionsQuery();
    $results = $query->execute()->fetchAll();
    foreach ($results as $result) {

      // Check if entity exists.
      $entity = $this->entityTypeManager->getStorage('origin_field_collection')->load($result->field_name);
      if (!$entity) {
        $entity = $this->entityTypeManager->getStorage('origin_field_collection')->create([
          'id' => $result->field_name,
          'label' => $result->field_name,
          'description' => $result->field_name,
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginParagraphs(): void
  {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_paragraph')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_paragraph')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function readOriginParagraphs(): void
  {
    $results = $this->getParagraphsQuery();
    foreach ($results as $result) {

      // Check if entity exists.
      $entity = $this->entityTypeManager->getStorage('origin_paragraph')->load($result->getSourceProperty('bundle'));
      if (!$entity) {
        $entity = $this->entityTypeManager->getStorage('origin_paragraph')->create([
          'id' => $result->getSourceProperty('bundle'),
          'label' => $result->getSourceProperty('bundle'),
          'description' => $result->getSourceProperty('bundle'),
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginDefaultLangcode($current_config = NULL) {
    if($current_config){
      $this->config->set('migrate_wizard_settings', $current_config)->save();
    }

    $query = $this->select('variable', 'v')
      ->fields('v', ['value'])
      ->condition('v.name', 'language_default');
    $results_query = $query->execute()->fetchField();
    //unserialize the data
    $results = unserialize($results_query);
    return $results->language;
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginLanguages($origin_type, $origin_bundle): array
  {

    $results['origin_languages'] = [];
    switch($origin_type){
      case 'node':
        $query_language_content_type = $this->select('variable', 'v')
          ->fields('v', ['value'])
          ->condition('v.name', 'language_content_type_'.$origin_bundle);

        $results_language_content_type = $query_language_content_type->execute()->fetchField();
        $language_content_type = unserialize($results_language_content_type);

        if($language_content_type>0){
          $results['origin_languages'] = $this->getLanguageTranslations();
        }
        break;
      case 'taxonomy_term':
        $query_type_i18n_mode = $this->select('taxonomy_vocabulary', 'tv')
          ->fields('tv', ['i18n_mode', 'language'])
          ->condition('tv.machine_name', $origin_bundle);
        $query_type_i18n_mode = $query_type_i18n_mode->execute()->fetchAll();
        $results['i18n_mode'] = $query_type_i18n_mode[0]['i18n_mode'];
        $results['language'] = $query_type_i18n_mode[0]['language'];
        if($results['i18n_mode'] != 0){
          $results['origin_languages'] = $this->getLanguageTranslations();
        }
        break;
      case 'field_collection_item':
        $results['origin_languages'] = $this->getLanguageTranslations();
        break;
    }

    return $results;
  }

  private function getLanguageTranslations(): array
  {
    $default_langcode = $this->getOriginDefaultLangcode();
    $query = $this->select('languages', 'l')
      ->fields('l', ['language'])
      ->condition('l.enabled', '0', '!=')
      ->condition('l.language', $default_langcode, '!=');
    $results_query = $query->execute()->fetchAll();
    $results = [];
    foreach ($results_query as $language) {
      $results[] = $language['language'];
    }
    return $results;
  }

  public function getDatabase() {
    if (!isset($this->database)) {
      $this->database = $this->setUpDatabase([]);
    }
    return $this->database;
  }

  //similar to getfields, but includes fc.data
  public function getFieldsOriginBundle($origin_bundle, $originType): array
  {
    $query = $this->select('field_config_instance', 'fci')
      ->fields('fci', ['field_id'])
      ->fields('fci', ['field_name'])
      ->fields('fci', ['data'])
      ->condition('fci.entity_type', $originType)
      ->condition('fci.bundle', $origin_bundle)
      ->condition('fci.deleted', 0);

    $query->leftJoin('field_config', 'fc', "fci.field_id = fc.id");

    $query->fields("fc", [
      'type',
      'data',
    ]);

    return $query->execute()->fetchAll();
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginEditorFormats(): array
  {
    $query = $this->select('ckeditor_input_format', 'cif')
      ->fields('cif', ['name'])
      ->fields('cif', ['format']);
    $results_query = $query->execute()->fetchAll();
    return $results_query;
  }

  /**
   * {@inheritdoc}
   */
  public function getCurrentContentTypes(): array
  {

    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    foreach ($content_types as $content_type) {
      $content_types_array[$content_type->id()] = $content_type->label();
    }
    return $content_types_array;
  }

  /**
   * {@inheritdoc}
   */
  public function getCurrentVocabularies(): array
  {

    // Get all vocabularies.
    $vocabularies = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->loadMultiple();
    foreach ($vocabularies as $vocabularie) {
      $vocabularies_array[$vocabularie->id()] = $vocabularie->label();
    }
    return $vocabularies_array;
  }

  /**
   * {@inheritdoc}
   */
  public function getParagraphBundles(): array
  {
    $paragraph_types = $this->entityTypeManager->getStorage('paragraphs_type')->loadMultiple();
    $paragraph_types_array = [];
    foreach ($paragraph_types as $paragraph_type) {
      $paragraph_types_array[$paragraph_type->id()] = $paragraph_type->label();
    }
    return $paragraph_types_array;
  }
  /**
   * {@inheritdoc}
   */
  public function getOriginRoles(): array
  {
    $query = $this->select('role', 'r')
      ->fields('r', ['name'])
      ->fields('r', ['rid']);

    $roles = $query->execute()->fetchAll();

    $roles_array = [];
    foreach ($roles as $rol) {
      $roles_array[$rol->rid] = $rol->name;
    }
    return $roles_array;
  }

  public function getParagraphFieldsByBundle($bundle): array
  {
    $query = $this->select('paragraphs_item', 'pi')
      ->fields('pi', ['item_id'])
      ->fields('pi', ['bundle'])
      ->condition('pi.field_name', $bundle);

    $query->leftJoin('field_config_instance', 'fci', "pi.bundle = fci.bundle");
    $query->fields('fci', ['field_id'])
      ->fields('fci', ['field_name'])
      ->fields('fci', ['data']);

    $query->leftJoin('field_config', 'fc', "fci.field_id = fc.id");

    $query->fields("fc", [
      'type',
      'data',
    ]);

    return $query->execute()->fetchAll();
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldsMedia(): array
  {
    $fieldsArrayMedia = [];
    $media_types = $this->entityTypeManager->getListBuilder('media_type')->load();
    foreach ($media_types as $key => $media_type) {
      $fieldMediaDefinitions = $this->entityFieldManager->getFieldDefinitions('media', $key);
      $fieldMediaBaseDefinitions = $this->entityFieldManager->getBaseFieldDefinitions('media');
      $fieldsExtraMedia = array_diff_key($fieldMediaDefinitions, $fieldMediaBaseDefinitions);
      foreach ($fieldsExtraMedia as $fieldExtraMedia) {
        if ($fieldExtraMedia->getSetting('target_type') == 'file' && !isset($fieldsArrayMedia[$fieldExtraMedia->getName()])) {
          $fieldsArrayMedia[$fieldExtraMedia->getName()] = $fieldExtraMedia->getLabel();
        }
      }
    }
    return $fieldsArrayMedia;
  }

  /**
   * {@inheritdoc}
   */
  public function createRoles($array_config): void
  {
    foreach ($array_config["roles"] as &$role) {
      if ($role['destiny'] == 'create') {
        $data = [
          'id' => $this->transliterateString($role['origin']),
          'label' => $role['origin'],
        ];
        // Creating your role.
        $role_obj = Role::create($data);
        // Saving your role.
        $role_obj->save();
        $role['destiny'] = $role_obj->id();

      }
    }
    $this->config->set('origin_user_roles', $array_config)->save();
  }

  /**
   * Transforms string to machine name format
   *
   * @param string $input
   * @return string|null
   */
  public function transliterateString(string $input): ?string
  {
    $transliterated = \Drupal::transliteration()->transliterate($input, LanguageInterface::LANGCODE_DEFAULT, '_');
    $transliterated = mb_strtolower($transliterated);
    $transliterated = preg_replace('@[^a-z0-9_./\s+/]+@', '_', $transliterated);
    return str_replace(' ', '_', $transliterated);
  }

  public function checkIfCasModuleOriginExistsEnabled(){
    $query = $this->select('system', 's')
      ->fields('s', ['status'])
      ->condition('s.name', 'cas');
    return $query->execute()->fetchField();
  }

  public function getOriginBundleTitle($bundle, $type){
    switch($type){
      case 'node':
        $query = $this->select('node_type', 'nt')
          ->fields('nt', ['name'])
          ->condition('nt.type', $bundle);
        break;
      case 'taxonomy_term':
        $query = $this->select('taxonomy_vocabulary', 'tv')
          ->fields('tv', ['name'])
          ->condition('tv.machine_name', $bundle);
        break;
      case 'user':
        $query = $this->select('role', 'r')
          ->fields('r', ['name'])
          ->condition('r.rid', $bundle);
        break;
    }
    return $query->execute()->fetchField();
  }

  //check if table exits
  public function checkWorkbenchAccessNodeExist(){
    $query = $this->select('system', 's')
      ->fields('s', ['status'])
      ->condition('s.status', 1)
      ->condition('s.name', 'workbench_access');
    return $query->execute()->fetchField();
  }

  public function fields()
  {
    // TODO: Implement fields() method.
  }

  public function getIds()
  {
    // TODO: Implement getIds() method.
  }

  public function query()
  {
    // TODO: Implement query() method.
  }
}

==> migrate_wizard-1.0.17-alpha1.tar.gz: migrate_wizard/src/MWManageDataService.php <==
<?php

namespace Drupal\migrate_wizard;

use Drupal\Core\Language\LanguageInterface;
use Drupal\migrate\Plugin\MigrationDeriverTrait;
use Drupal\migrate_drupal\Plugin\migrate\source\d7\FieldableEntity;
use Drupal\user\Entity\Role;
use Drupal\Component\Transliteration\PhpTransliteration;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ConfigManager;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ExtensionList;

/**
 * Defines a Service to update nodes.
 */
class MWManageDataService extends FieldableEntity{

  /**
   * {@inheritdoc}
   */
  protected $entityType;

  /**
   * {@inheritdoc}
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected $migrationUrlFilesProd;

  /**
   * {@inheritdoc}
   */
  protected $extensionListModule;

  /**
   * {@inheritdoc}
   */
  protected $configManager;

  /**
   * {@inheritdoc}
   */
  protected $transliterator;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager,
                              EntityTypeManagerInterface $entity_type_manager,
                              ConfigFactoryInterface $configFactory,
                              ExtensionList $extension_list_module,
                              ConfigManager $config_manager,
                              PhpTransliteration $php_transliteration) {

    $this->config = $configFactory->getEditable('migrate_wizard.settings');
    if (is_array($this->config->get('migrate_wizard_settings'))) {
      $this->migrationUrlFilesProd = $this->config->get('migrate_wizard_settings')['migration_url_files_prod'];
    }
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->extensionListModule = $extension_list_module;
    $this->configManager = $config_manager;
    $this->transliterator = $php_transliteration;
  }

  use MigrationDeriverTrait;

  /**
   * {@inheritdoc}
   */
  protected function getNodeTypesQuery() {
    return static::getSourcePlugin('d7_node_type');
  }

  /**
   * {@inheritdoc}
   */
  protected function getVocabulariesQuery() {
    return static::getSourcePlugin('d7_taxonomy_vocabulary');
  }

  /**
   * {@inheritdoc}
   */
  protected function getFieldCollectionsQuery() {
    $query = $this->select('field_config', 'fc')
      ->fields('fc', ['field_name'])
      ->groupBy('fc.field_name')
      ->condition('fc.type', 'field_collection');
    return $query;
  }

  /**
   * {@inheritdoc}
   */
  protected function getParagraphsQuery() {
    return static::getSourcePlugin('d7_paragraphs_item');
  }
  /**
   * {@inheritdoc}
   */
  public function readOriginNodes(): void
  {
    $results = $this->getNodeTypesQuery();
    foreach ($results as $result) {
      // Check if entity exists.
      $entity = $this->entityTypeManager->getStorage('origin_node')->load($result->getSourceProperty('type'));
      if (!$entity) {
        $entity = $this->entityTypeManager->getStorage('origin_node')->create([
          'id' => $result->getSourceProperty('type'),
          'label' => $result->getSourceProperty('name'),
          'description' => $result->getSourceProperty('description'),
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginNodes(): void
  {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_node')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_node')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginVocabularies(): void
  {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_vocabulary')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_vocabulary')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function readOriginVocabularies(): void
  {
    $results = $this->getVocabulariesQuery();

    foreach($results as $result){
     $entity = $this->entityTypeManager->getStorage('origin_vocabulary')->load($result->getSourceProperty('machine_name'));
      if ($entity == NULL) {
        $entity = $this->entityTypeManager->getStorage('origin_vocabulary')->create([
          'id' => $result->getSourceProperty('machine_name'),
          'label' => $result->getSourceProperty('name'),
          'description' => $result->getSourceProperty('description'),
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginFieldCollections(): void
  {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_field_collection')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_field_collection')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function readOriginFieldCollections(): void
  {
    $query = $this->getFieldCollectionsQuery();
    $results = $query->execute()->fetchAll();
    foreach ($results as $result) {

      // Check if entity exists.
      $entity = $this->entityTypeManager->getStorage('origin_field_collection')->load($result->field_name);
      if (!$entity) {
        $entity = $this->entityTypeManager->getStorage('origin_field_collection')->create([
          'id' => $result->field_name,
          'label' => $result->field_name,
          'description' => $result->field_name,
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function clearOriginParagraphs(): void
  {
    // Delete all entities of type origin_node.
    $entity = $this->entityTypeManager->getStorage('origin_paragraph')->loadMultiple();
    $this->entityTypeManager->getStorage('origin_paragraph')->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function readOriginParagraphs(): void
  {
    $results = $this->getParagraphsQuery();
    foreach ($results as $result) {

      // Check if entity exists.
      $entity = $this->entityTypeManager->getStorage('origin_paragraph')->load($result->getSourceProperty('bundle'));
      if (!$entity) {
        $entity = $this->entityTypeManager->getStorage('origin_paragraph')->create([
          'id' => $result->getSourceProperty('bundle'),
          'label' => $result->getSourceProperty('bundle'),
          'description' => $result->getSourceProperty('bundle'),
          'source_type' => 'sql',
          'module' => 'migrate_wizard',
          'shared_configuration' => 0,
        ]);
        $entity->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginDefaultLangcode($current_config = NULL) {
    if($current_config){
      $this->config->set('migrate_wizard_settings', $current_config)->save();
    }

    $query = $this->select('variable', 'v')
      ->fields('v', ['value'])
      ->condition('v.name', 'language_default');
    $results_query = $query->execute()->fetchField();
    //unserialize the data
    $results = unserialize($results_query);
    return $results->language;
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginLanguages($origin_type, $origin_bundle): array
  {

    $results['origin_languages'] = [];
    switch($origin_type){
      case 'node':
        $query_language_content_type = $this->select('variable', 'v')
          ->fields('v', ['value'])
          ->condition('v.name', 'language_content_type_'.$origin_bundle);

        $results_language_content_type = $query_language_content_type->execute()->fetchField();
        $language_content_type = unserialize($results_language_content_type);

        if($language_content_type>0){
          $results['origin_languages'] = $this->getLanguageTranslations();
        }
        break;
      case 'taxonomy_term':
        $query_type_i18n_mode = $this->select('taxonomy_vocabulary', 'tv')
          ->fields('tv', ['i18n_mode', 'language'])
          ->condition('tv.machine_name', $origin_bundle);
        $query_type_i18n_mode = $query_type_i18n_mode->execute()->fetchAll();
        $results['i18n_mode'] = $query_type_i18n_mode[0]['i18n_mode'];
        $results['language'] = $query_type_i18n_mode[0]['language'];
        if($results['i18n_mode'] != 0){
          $results['origin_languages'] = $this->getLanguageTranslations();
        }
        break;
      case 'field_collection_item':
        $results['origin_languages'] = $this->getLanguageTranslations();
        break;
    }

    return $results;
  }

  private function getLanguageTranslations(): array
  {
    $default_langcode = $this->getOriginDefaultLangcode();
    $query = $this->select('languages', 'l')
      ->fields('l', ['language'])
      ->condition('l.enabled', '0', '!=')
      ->condition('l.language', $default_langcode, '!=');
    $results_query = $query->execute()->fetchAll();
    $results = [];
    foreach ($results_query as $language) {
      $results[] = $language['language'];
    }
    return $results;
  }

  public function getDatabase() {
    if (!isset($this->database)) {
      $this->database = $this->setUpDatabase([]);
    }
    return $this->database;
  }

  //similar to getfields, but includes fc.data
  public function getFieldsOriginBundle($origin_bundle, $originType): array
  {
    $query = $this->select('field_config_instance', 'fci')
      ->fields('fci', ['field_id'])
      ->fields('fci', ['field_name'])
      ->fields('fci', ['data'])
      ->condition('fci.entity_type', $originType)
      ->condition('fci.bundle', $origin_bundle)
      ->condition('fci.deleted', 0);

    $query->leftJoin('field_config', 'fc', "fci.field_id = fc.id");

    $query->fields("fc", [
      'type',
      'data',
    ]);

    return $query->execute()->fetchAll();
  }

  /**
   * {@inheritdoc}
   */
  public function getOriginEditorFormats(): array
  {
    $query = $this->select('filter_format', 'ff')
      ->fields('ff', ['name'])
      ->fields('ff', ['format']);
    $results_query = $query->execute()->fetchAll();
    return $results_query;
  }

  /**
   * {@inheritdoc}
   */
  public function getCurrentContentTypes(): array
  {

    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    foreach ($content_types as $content_type) {
      $content_types_array[$content_type->id()] = $content_type->label();
    }
    return $content_types_array;
  }

  /**
   * {@inheritdoc}
   */
  public function getCurrentVocabularies(): array
  {

    // Get all vocabularies.
    $vocabularies = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->loadMultiple();
    foreach ($vocabularies as $vocabularie) {
      $vocabularies_array[$vocabularie->id()] = $vocabularie->label();
    }
    return $vocabularies_array;
  }

  /**
   * {@inheritdoc}
   */
  public function getParagraphBundles(): array
  {
    $paragraph_types = $this->entityTypeManager->getStorage('paragraphs_type')->loadMultiple();
    $paragraph_types_array = [];
    foreach ($paragraph_types as $paragraph_type) {
      $paragraph_types_array[$paragraph_type->id()] = $paragraph_type->label();
    }
    return $paragraph_types_array;
  }
  /**
   * {@inheritdoc}
   */
  public function getOriginRoles(): array
  {
    $query = $this->select('role', 'r')
      ->fields('r', ['name'])
      ->fields('r', ['rid']);

    $roles = $query->execute()->fetchAll();

    $roles_array = [];
    foreach ($roles as $rol) {
      $roles_array[$rol->rid] = $rol->name;
    }
    return $roles_array;
  }

  public function getParagraphFieldsByBundle($bundle): array
  {
    $query = $this->select('paragraphs_item', 'pi')
      ->fields('pi', ['item_id'])
      ->fields('pi', ['bundle'])
      ->condition('pi.field_name', $bundle);

    $query->leftJoin('field_config_instance', 'fci', "pi.bundle = fci.bundle");
    $query->fields('fci', ['field_id'])
      ->fields('fci', ['field_name'])
      ->fields('fci', ['data']);

    $query->leftJoin('field_config', 'fc', "fci.field_id = fc.id");

    $query->fields("fc", [
      'type',
      'data',
    ]);

    return $query->execute()->fetchAll();
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldsMedia(): array
  {
    $fieldsArrayMedia = [];
    $media_types = $this->entityTypeManager->getListBuilder('media_type')->load();
    foreach ($media_types as $key => $media_type) {
      $fieldMediaDefinitions = $this->entityFieldManager->getFieldDefinitions('media', $key);
      $fieldMediaBaseDefinitions = $this->entityFieldManager->getBaseFieldDefinitions('media');
      $fieldsExtraMedia = array_diff_key($fieldMediaDefinitions, $fieldMediaBaseDefinitions);
      foreach ($fieldsExtraMedia as $fieldExtraMedia) {
        if ($fieldExtraMedia->getSetting('target_type') == 'file' && !isset($fieldsArrayMedia[$fieldExtraMedia->getName()])) {
          $fieldsArrayMedia[$fieldExtraMedia->getName()] = $fieldExtraMedia->getLabel();
        }
      }
    }
    return $fieldsArrayMedia;
  }

  /**
   * {@inheritdoc}
   */
  public function createRoles($array_config): void
  {
    foreach ($array_config["roles"] as &$role) {
      if ($role['destiny'] == 'create') {
        $data = [
          'id' => $this->transliterateString($role['origin']),
          'label' => $role['origin'],
        ];
        // Creating your role.
        $role_obj = Role::create($data);
        // Saving your role.
        $role_obj->save();
        $role['destiny'] = $role_obj->id();

      }
    }
    $this->config->set('origin_user_roles', $array_config)->save();
  }

  /**
   * Transforms string to machine name format
   *
   * @param string $input
   * @return string|null
   */
  public function transliterateString(string $input): ?string
  {
    $transliterated = \Drupal::transliteration()->transliterate($input, LanguageInterface::LANGCODE_DEFAULT, '_');
    $transliterated = mb_strtolower($transliterated);
    $transliterated = preg_replace('@[^a-z0-9_./\s+/]+@', '_', $transliterated);
    return str_replace(' ', '_', $transliterated);
  }

  public function checkIfCasModuleOriginExistsEnabled(){
    $query = $this->select('system', 's')
      ->fields('s', ['status'])
      ->condition('s.name', 'cas');
    return $query->execute()->fetchField();
  }

  public function getOriginBundleTitle($bundle, $type){
    switch($type){
      case 'node':
        $query = $this->select('node_type', 'nt')
          ->fields('nt', ['name'])
          ->condition('nt.type', $bundle);
        break;
      case 'taxonomy_term':
        $query = $this->select('taxonomy_vocabulary', 'tv')
          ->fields('tv', ['name'])
          ->condition('tv.machine_name', $bundle);
        break;
      case 'user':
        $query = $this->select('role', 'r')
          ->fields('r', ['name'])
          ->condition('r.rid', $bundle);
        break;
    }
    return $query->execute()->fetchField();
  }

  //check if table exits
  public function checkWorkbenchAccessNodeExist(){
    $query = $this->select('system', 's')
      ->fields('s', ['status'])
      ->condition('s.status', 1)
      ->condition('s.name', 'workbench_access');
    return $query->execute()->fetchField();
  }

  public function fields()
  {
    // TODO: Implement fields() method.
  }

  public function getIds()
  {
    // TODO: Implement getIds() method.
  }

  public function query()
  {
    // TODO: Implement query() method.
  }
}
